import 'dart:async';
import 'dart:collection';
import 'package:flutter/material.dart';
import 'package:btlib_example/widgets.dart';
import 'package:btlib/btlib.dart';

Btlib driver = Btlib();

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Btlib Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(title: 'Btlib Demo Home Page'),
    );
  }
}

class MemoMessage extends StatelessWidget {
  MemoMessage({this.text});
  final String text;
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: const EdgeInsets.symmetric(vertical: 0.0),
      child: Text(
        text,
        style: DefaultTextStyle.of(context).style.apply(fontSizeFactor: 0.8),
      ),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key key, this.title}) : super(key: key);
  final String title;
  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  final List<MemoMessage> memo = <MemoMessage>[];
  StreamSubscription subscription;
  Map<String, BTDevice> devices = new Map();
  bool scanning = false;

  @override
  void initState() {
    super.initState();
    subscription = driver.listen(onEvent);
  }

  void log(String msg) {
    setState(() {
      memo.insert(0, MemoMessage(text:msg));
      if (memo.length > 255) memo.removeLast();
    });
  }

  @override
  void dispose() {
    subscription.cancel();
    super.dispose();
  }

  void onEvent(Object e) {
    HashMap<String, Object> map = HashMap.from(e);
    print(map);
    var event           = map["event"] != null ? map["event"].toString() : "";
    var name            = map["name"] != null ? map["name"].toString() : "";
    var address         = map["address"] != null ? map["address"].toString() : "";
    var service         = map["service"] != null ? map["service"].toString() : "";
    var characteristic  = map["characteristic"] != null ? map["characteristic"].toString() : "";
    var data            = map["data"] != null ? map["data"].toString() : "";
    var rssi            = map["rssi"] != null ? map["rssi"].toString() : "";
    log(event);
    if (event == BTEvent.BLE_START_SCAN) {
      devices.clear();
      scanning = true;
    } else if (event == BTEvent.BLE_STOP_SCAN) {
      scanning = false;
    } else if (event == BTEvent.BLE_CONNECTED) {
      devices[address].connected = true;
    } else if (event == BTEvent.BLE_DISCONNECTED) {
      devices[address].connected = false;
    } else if (event == BTEvent.BLE_SCAN_RESULT) {
      devices[address] = BTDevice(name, address, false);
      log("["+name+"]["+address+"]["+rssi+"]");
    } else if (event == BTEvent.BLE_DATA_AVAILABLE) {
      log("["+address+"]["+service+"]["+characteristic+"]"+"["+data+"]");
    } else if (event == BTEvent.BLE_SERVICES_CHARACTERISTICS) {
      if( devices[address].services[service] == null )
      {
        devices[address].services[service] = BTService(devices[address], service);
      }
      devices[address].services[service].characteristics[characteristic] = BTCharacteristic(devices[address].services[service], characteristic);
      log("["+address+"]["+service+"]["+characteristic+"]");
    } else if (event == BTEvent.BLE_RSSI) {
      log("["+address+"]["+ rssi +"]");
    }
  }

  _buildDeviceTiles() {
    return devices.values
        .map((device) => DeviceTile( device: device ))
        .toList();
  }

  _buildScanningButton() {
    if (scanning) {
      return new FloatingActionButton(
        child: new Icon(Icons.stop),
        onPressed: () {
          driver.stopScan();
        },
        backgroundColor: Colors.red,
      );
    } else {
      return new FloatingActionButton(
        child: new Icon(Icons.search),
        onPressed: () {
          driver.startScan();
        },
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    var tiles = new List<Widget>();
    tiles.addAll(_buildDeviceTiles());

    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
      ),
      floatingActionButton: _buildScanningButton(),
      body: Column(
        children: <Widget>[
          Expanded(
              // wrap in Expanded
              flex: 75,
              child: ListView(
                padding: const EdgeInsets.all(8.0),
                children: tiles,
              )),
          Flexible(
            flex: 25,
            child: ListView.builder(
              padding: EdgeInsets.all(8.0),
              reverse: true,
              itemBuilder: (_, int index) => memo[index],
              itemCount: memo.length,
            ),
          ),
        ],
      ),
    );
  }
}
